/*
 * MapKey.java
 * Created on 2011/10/13
 * RCSID: $Id:$
 *
 * Copyright (C) 2011-2013 Nippon Telegraph and Telephone Corporation
 *
 */

package jp.co.ntt.oss.heapstats.container;

/**
 * implements {@link Comparable}<br>
 * <br>
 * Holds one pair of tags and class path.
 */
public class MapKey implements
                   Comparable<MapKey> {
    /** Class Tag. */
    private long tag;

    /** Class Path. */
    private String name;

    /**
     * Creates a MapKey.
     *
     * @param source ObjectData
     */
    public MapKey(final ObjectData source) {
        tag = source.getTag();
        name = source.getName();
    }

    /**
     * Creates a MapKey.
     *
     * @param tagValue the tag
     * @param nameValue the Class path
     */
    public MapKey(final long tagValue, final String nameValue) {
        tag = tagValue;
        name = nameValue;
    }

    /**
     * Sets the tag.
     *
     * @param value tag
     */
    public final void setTag(final long value) {
        tag = value;
    }

    /**
     * Return the tag.
     *
     * @return Return the tag
     */
    public final long getTag() {
        return tag;
    }

    /**
     * Sets the Class path.
     *
     * @param value Class path
     */
    public final void setName(final String value) {
        name = value;
    }

    /**
     * Return the Class Name.
     *
     * @return Return the Class Name
     */
    public final String getName() {
        return name;
    }

    @Override
    public final String toString() {
        return name;
    }

    @Override
    public final int compareTo(final MapKey key) {
        long ret = tag - key.tag;

        if (ret == 0) {
            return 0;
        } else if (ret < 0) {
            return -1;
        } else {
            return 1;
        }
    }

    @Override
    public final boolean equals(final Object obj) {
        if (obj == null) {
            return false;
        }
        if (obj instanceof MapKey) {
            return ((MapKey) obj).tag == tag;
        }
        return false;
    }

    @Override
    public final int hashCode() {
        return Long.valueOf(tag).hashCode();
    }
}
