/*
 * PackageFilter.java
 * Created on 2011/10/13
 * RCSID: $Id:$
 *
 * Copyright (C) 2011-2013 Nippon Telegraph and Telephone Corporation
 *
 */

package jp.co.ntt.oss.heapstats.container;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import jp.co.ntt.oss.heapstats.gui.FilterEntriesModel;

/**
 * Holds the path to the excluded class.
 */
public class PackageFilter {
    /** Class Filter Map. Key is Filter name. Data is visible and class path. */
    private Map<String, List<String>> classFilter;

    /**
     * Create a PackageFilter.
     */
    public PackageFilter() {
        classFilter = new HashMap<String, List<String>>();
    }

    /**
     * Add a filter definition from the file read.
     *
     * @param addFilter Filter definition information to be added
     * @param filterEntriesModel FilterEntriesModel to draw the filter
     *        definition
     * @return Additional results filter definition. true is added.
     */
    public final boolean add(final Map<String, List<String>> addFilter,
            final FilterEntriesModel filterEntriesModel) {
        if(addFilter == null){
            return true;
        }
        Set<String> keys = addFilter.keySet();
        boolean duplicate = false;
        for (String key : keys) {
            List<String> data = classFilter.get(key);
            if (data != null) {
                duplicate = !duplicate;
                break;
            }
        }

        if (!duplicate) {
            Object[] rows = new Object[2];
            for (String key : keys) {
                rows[0] = Boolean.valueOf(addFilter.get(key).get(0));
                rows[1] = key;
                filterEntriesModel.addRow(rows);
            }

            classFilter.putAll(addFilter);

            return true;
        }
        return false;
    }

    /**
     * Clear filter definition.
     */
    public final void clear() {
        classFilter.clear();
    }

    /**
     * Returns the class path to be filtered.
     *
     * @param key Filter name
     * @return Returns the class path to be filtered.
     */
    public final List<String> get(final String key) {
        return classFilter.get(key);
    }
}
