/*
 * GuiCommon.java
 * Created on 2011/09/08
 * RCSID: $Id: GuiCommon.java 407 2012-05-21 00:30:29Z alpha_takahiro.jyoukou $
 *
 * Copyright (C) 2011-2013 Nippon Telegraph and Telephone Corporation
 *
 */

package jp.co.ntt.oss.heapstats.gui;

import java.awt.Component;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;

import javax.swing.JOptionPane;

/**
 * Define a common resource to be used in heapstats.
 */
public final class GuiCommon {

    /** Size column lists the object's color. */
    public static final int COL_SIZE_COLOR = 10;

    /** Size column lists the class name. */
    public static final int COL_SIZE_CLASSNAME = 200;

    /** Binary prefix. */
    public static final int BINARY_PREFIX = 1024;

    /** Menu-File key. */
    public static final String MENU_FILE = "file";

    /** Menu-File-Log File Open key. */
    public static final String MENU_FILE_LOGOPEN = "logopen";

    /** Menu-File-Open key. */
    public static final String MENU_FILE_OPEN = "open";

    /** Menu-File-Exit key. */
    public static final String MENU_FILE_EXIT = "exit";

    /** Menu-Option key. */
    public static final String MENU_OPTION = "option";

    /** Menu-Option-Tab key. */
    public static final String MENU_OPTION_TAB = "tabbar";

    /** Menu-Option-Ranking Graph Level key. */
    public static final String MENU_OPTION_RANKLEVEL = "rank";

    /** Menu-Option-Write to CSV key. */
    public static final String MENU_OPTION_CSV = "csv";

    /** Menu-Option-CSV-Heap Usage key. */
    public static final String MENU_OPTION_CSVHEAP = "csvheap";

    /** Menu-Option-CSV-GC statistics key. */
    public static final String MENU_OPTION_CSVGC = "csvstatistics";

    /** Menu-Option-CSV-all key. */
    public static final String MENU_OPTION_CSVALL = "csvall";

    /** Menu-Option-CSV-selected key. */
    public static final String MENU_OPTION_CSVSELECTED = "csvselected";

    /** Menu-help key. */
    public static final String MENU_HELP = "help";

    /** Menu-help-how to use key. */
    public static final String MENU_USE = "howtouse";

    /** Menu-help-version key. */
    public static final String MENU_HELP_VERSION = "version";

    /** Window title for the version key. */
    public static final String VERSION_DIALOG_TITLE = "versiontitle";

    /** Manual for simple text file name. */
    public static final String HOWTO_TEXT = "howtotext";

    /** The key to get the tab title of Statistics GC. */
    public static final String TAB_TITLE_SUMMARY = "tabsummary";

    /** The key to get the tab title of Delta. */
    public static final String TAB_TITLE_DELA = "tabdelta";

    /** The key to get the tab title of SnapShot. */
    public static final String TAB_TITLE_SNAPSHOT = "tabsnapshot";

    /** The key to get the tab title of Resource. */
    public static final String TAB_TITLE_RESOUCE = "tabresouce";

    /** The key to get the tab title of Log Collect. */
    public static final String TAB_TITLE_LOG = "tablogcollection";

    /** Ranking Settings dialog label title. */
    public static final String RANK_LEVEL = "ranktitle";

    /** The key to get the tab title of Log Collect. */
    public static final String MAIN_TAB_LOGCOLLECT = "maintablog";

    /** The key to get the tab title of SnapShot. */
    public static final String MAIN_TAB_SNAPSHOT = "maintabsnapshot";

    /** Select Log Main Title. */
    public static final String TITLE_LOG = "titellog";

    /** LOG SELECT Button. */
    public static final String BUTTON_LOG_SELECT = "logselectbutton";

    // SELECT DATA
    /** SELECT DATA MAIN TITLE. */
    public static final String TITLE_MAIN = "titlemain";

    /** SELECT DATA BUTTON. */
    public static final String SELECT_DATA = "selectdata";

    /** SELECT RAGE LABEL. */
    public static final String SELECT_RANGE = "selectrange";

    // Summary Data
    /** The accordion panel title of the chart statistics. */
    public static final String TITLE_STATISTICS = "titlestatistics";

    /** the row key. */
    public static final String TOTAL_HEAP_SIZE = "totalheapsize";

    /** the row key. */
    public static final String NEW_HEAP = "newheapsize";

    /** the row key. */
    public static final String OLD_HEAP = "oldheapsize";

    /** the row key. */
    public static final String GCTIME = "gctime";

    /** the table key. */
    public static final String TITLE_SUMMARY = "titlesummary";

    /** the table key. */
    public static final String SNAPCOUNT = "snapcount";

    /** the table key. */
    public static final String GCCOUNT = "gccount";

    /** the table key. */
    public static final String LATEST_HEAP = "latestheapusage";

    /** the table key. */
    public static final String MIN_GCTIME = "mingctime";

    /** the table key. */
    public static final String MAX_GCTIME = "maxgctime";

    // Delta Data
    /** The accordion panel title of the chart ranking(size). */
    public static final String TITLE_SIZE_DELTE = "titledeltasize";

    /** The accordion panel title of the chart ranking(instance). */
    public static final String TITLE_INSTANCE_DELTE = "titledeltainstance";

    /** The accordion panel title of the chart delta(size). */
    public static final String TITLE_SIZE_AREA = "titleareachartsize";

    /** The accordion panel title of the chart delta(instance). */
    public static final String TITLE_INSTANCE_AREA = "titleareachartinstance";

    /** The accordion panel title of the objects list. */
    public static final String TITLE_OBJECTLIST = "titleobjectslist";

    /** the table title of the object name. */
    public static final String OBJECTLIST_NAME = "name";

    /** the table title of the object instances. */
    public static final String OBJECTLIST_INSTANCE = "instances";

    /** the table title of the heap usage. */
    public static final String OBJECTLIST_SIZE = "size";

    /** Class Filter Label title. */
    public static final String CLASS_FILTER_LABEL_TITLE = "titlefilter";

    /** Class Filter ADD Button. */
    public static final String CLASS_FILTER_ADD = "addfilter";

    /** Class Search Label title. */
    public static final String CLASS_SEARCH_LABEL_TITLE = "titlesearch";

    /** Exclude Key. */
    public static final String CLASS_FILTER_EXCLUDE = "exclude";

    /** Filter Name Key. */
    public static final String CLASS_FILTER_FILENAME = "filtername";

    /** Sort button(instance). */
    public static final String SORT_INSTANCE = "sortinstance";

    /** Sort button(heap usage). */
    public static final String SORT_SIZE = "sortsize";

    // SnapShot Data
    /** The accordion panel title of the SnapShot Data. */
    public static final String TITLE_SNAPSHOT = "titlesnapshot";

    /** The accordion panel title of the chart heap usage. */
    public static final String TITLE_PIECHART = "titlepiechart";

    /** the table key. */
    public static final String SNAPSHOTINFO_DATE = "date";

    /** the table key. */
    public static final String SNAPSHOTINFO_ENTRYIES = "entries";

    /** the table key. */
    public static final String SNAPSHOTINFO_HEAP = "totalheapusage";

    /** the table key. */
    public static final String SNAPSHOTINFO_CAUSE = "sscause";

    /** the table key. */
    public static final String SNAPSHOTINFO_GCCAUSE = "gccause";

    /** The format of the time you draw on screen. */
    public static final String DATE_FORMAT = "yyyyMMdd HH:mm:ss";

    /** The format of the time you draw on screen. */
    public static final String CHART_DATE_FORMAT = "MMdd HH:mm:ss.SSS";

    // Log Data
    /** The key to get the tab title of Resource. */
    public static final String TAB_RESOURCE_SUMMARY = "tabresource";

    /** The key to get the tab title of LogDetail. */
    public static final String TAB_LOG_DETAIL = "tablogdetail";

    /** The accordion panel title of the Resource reports. */
    public static final String TITLE_RESOURCE_REPORT = "titleresource";

    /** The accordion panel title of the Process reports. */
    public static final String TITLE_THREAD_SUMMARY = "processreports";

    /** The accordion panel title of the Resource reports. */
    public static final String TITLE_RESOURCE = "titleresouce";

    /** Name javaCPU chart. */
    public static final String GRAPH_JAVA_CPU = "graphjavacpu";

    /** Name systemCPU chart. */
    public static final String GRAPH_SYSTEM_CPU = "graphsyscpu";

    /** Name java Memory chart. */
    public static final String GRAPH_JAVA_MEMORY = "graphjavamem";

    /** Name threads chart. */
    public static final String GRAPH_THREADS = "graphthread";

    /** Field name of table. */
    public static final String ITEM_CPU_AVERAGE = "cpuave";

    /** Field name of table. */
    public static final String ITEM_CPU_PEAK = "cpupeak";

    /** Field name of table. */
    public static final String ITEM_VSZ_AVERAGE = "vszave";

    /** Field name of table. */
    public static final String ITEM_VSZ_PEAK = "vszpeak";

    /** Field name of table. */
    public static final String ITEM_RSS_AVERAGE = "rssave";

    /** Field name of table. */
    public static final String ITEM_RSS_PEAK = "rsspeak";

    /** The name of the Log Detail Data button. */
    public static final String NEXT_BUTTON = "nextbutton";

    /** The name of the Log Detail Data button. */
    public static final String PREV_BUTTON = "prevbutton";

    /** The Log Detail Data Table Header Name. */
    public static final String LOG_HEAD_ITEM = "tableheaditem";

    /** The Log Detail Data Table Header Name. */
    public static final String LOG_HEAD_DATA = "tableheaddata";

    /** POPUPMENU-CUT key. */
    public static final String POPUP_CUT = "popupcut";

    /** POPUPMENU-COPY key. */
    public static final String POPUP_COPY = "popupcopy";

    /** POPUPMENU-PASTE key. */
    public static final String POPUP_PASTE = "popuppaste";

    /** POPUPMENU-FIND key. */
    public static final String POPUP_FIND = "popupfind";

    // Find Dialog
    /** The key for obtain the title of the search dialog. */
    public static final String FIND_TITLE = "findtitle";

    /** The key for obtain the label of the search dialog. */
    public static final String FIND_LABEL = "findlabel";

    /** The key for obtain the name of the button of the search dialog. */
    public static final String FIND_NEXT = "findnext";

    /** The key for obtain the name of the button of the search dialog. */
    public static final String FIND_CANCEL = "findcancel";

    // Message
    /** Start the filter. */
    public static final String BASEI01 = "base-i-01";

    /** Complete output of the CSV. */
    public static final String BASEI02 = "base-i-02";

    /** "%s" string can not be found in the rear. */
    public static final String BASEI03 = "base-i-03";

    /** Failed to load heapstats.properties. Use the default values. */
    public static final String BASEW01 = "base-w-01";

    /** Please try again after waiting some time for processing. */
    public static final String BASEW02 = "base-w-02";

    /** Please input numeric value. */
    public static final String BASEE01 = "base-e-01";

    /** Please input positive value. */
    public static final String BASEE02 = "base-e-02";

    /** Please open SnapShot file. */
    public static final String BASEE03 = "base-e-03";

    /** Please specify the period. */
    public static final String BASEE05 = "base-e-05";

    /** Please select the time period again. */
    public static final String BASEE06 = "base-e-06";

    /** Start and end are the same. */
    public static final String BASEE07 = "base-e-07";

    /** Failed to load the file. */
    public static final String BASEE08 = "base-e-08";

    /** Couldn't find library directory. (%) */
    public static final String BASEE09 = "base-e-09";

    /** Duplicate filter name it, we canceled the loading. */
    public static final String BASEE10 = "base-e-10";

    /** An error occurred while loading a filter definition file. (%) */
    public static final String BASEE11 = "base-e-11";

    /** [SnapShot Data] failed to draw. */
    public static final String BASEE12 = "base-e-12";

    /** [Delta Data] failed to draw. */
    public static final String BASEE13 = "base-e-13";

    /** [Summary Data] failed to draw. */
    public static final String BASEE14 = "base-e-14";

    /** Failed to output CSV. [%s] */
    public static final String BASEE15 = "base-e-15";

    /** Error occurred during startup. Please reboot. */
    public static final String BASEE16 = "base-e-16";

    /** Failed to write the properties file. */
    public static final String BASEE17 = "base-e-17";

    /** Failed to open the file. */
    public static final String BASEE18 = "base-e-18";

    /** Failed to decode the file. */
    public static final String BASEE19 = "base-e-19";

    /** [Resource Data] failed to draw. */
    public static final String BASEE20 = "base-e-20";

    /** Default Frame Width. */
    public static final int DEFAULT_FRAME_WIDTH = 800;

    /** Default Frame Height. */
    public static final int DEFAULT_FRAME_HEIGHT = 600;

    /** Default width. */
    public static final int DEFAULT_DIALOG_WIDTH = 600;

    /** Default Height. */
    public static final int DEFAULT_DIALOG_HEIGHT = 400;

    /** Replacement character. */
    public static final String REPLACEMENT_CHAR = "%s";

    /** Analyzer window title. */
    public static final String ANALYZER_TITLE = "HeapStats Analyzer";

    /** Border size. */
    public static final int BORDER_SIZE = 5;

    /** Thread dump file name of the agent has collected. */
    public static final String THREAD_DUNM_FILE = "threaddump.txt";

    /** Split character to split the file path for multiple. */
    public static final String SPLIT_CHARACTER = ";";

    /** Default Language.Default is English. */
    private static final String DEFAULT_LANGUAGE = "en";

    /** Default Ranking Level.Default is 5. */
    private static final int DEFAULT_RANKING = 5;

    /** Rename the class path to the screen. Default is false. */
    private static final boolean DEFAULT_REPLACE = false;

    /** the HeapStatus property file name. */
    private static final String CONFIG_HEAPSTATS = "heapstats.properties";

    /** Default Language. */
    private static String language = DEFAULT_LANGUAGE;

    /** Default Ranking Level. */
    private static int rankLevel = DEFAULT_RANKING;

    /** the current directory. */
    private static File currentDir;

    /** Rename the class path to the screen. Renaming is true. */
    private static boolean classRename = DEFAULT_REPLACE;

    /** Rename the class path to the screen. Renaming is true. */
    private static String logfiles = "";

    /** Rename the class path to the screen. Renaming is true. */
    private static String socketend = "";

    /** Background color of the chart. */
    private static int bgColor = 0;

    /** Multilingual Resources. */
    private static ResourceBundle resources = ResourceBundle.getBundle(
            "HeapStatsResources", new Locale(language));

    /** The properties file heapstats. */
    private static Properties prop = new Properties();

    /**
     * Don't let anyone instantiate this class.
     */
    private GuiCommon() {
    }

    /**
     * Load the properties file.
     *
     * @throws Exception if the file does not exist, is a directory rather than
     *         a regular file, or for some other reason cannot be opened for
     *         reading.
     */
    public static void load() throws Exception {
        InputStream inputStream = null;
        try {
            File pp = new File(CONFIG_HEAPSTATS);
            inputStream = new FileInputStream(pp);
            prop.load(inputStream);
            language = prop.getProperty("language");
            if (language == null
                    || (!language.equals(DEFAULT_LANGUAGE) && !language
                            .equals("ja"))) {
                throw new Exception();
            }
            rankLevel = Integer.decode(prop.getProperty("ranklevel"));
            if (rankLevel < 1) {
                throw new Exception();
            }
            classRename = Boolean.parseBoolean(prop.getProperty("replace"));
            if (prop.getProperty("defaultdir") != null) {
                currentDir = new File(prop.getProperty("defaultdir"));
            }
            logfiles = prop.getProperty("logfile");
            socketend = prop.getProperty("socketend");
            bgColor = Integer.decode(prop.getProperty("bgcolor"));
        } catch (Exception e) {
            language = DEFAULT_LANGUAGE;
            rankLevel = DEFAULT_RANKING;
            classRename = DEFAULT_REPLACE;
            logfiles = "";
            socketend = "";
            currentDir = null;
            bgColor = 0;

            throw e;
        } finally {
            if (inputStream != null) {
                inputStream.close();
            }
        }
        resources = ResourceBundle.getBundle("HeapStatsResources", new Locale(
                language));
    }

    /**
     * Return the Language.
     *
     * @return Return the Language
     */
    public static String getLanguage() {
        return language;
    }

    /**
     * Return the Ranking level.
     *
     * @return Return the Ranking level.
     */
    public static int getRankLevel() {
        return rankLevel;
    }

    /**
     * Sets a Ranking level.
     *
     * @param rank ranking
     * @throws IOException if an I/O error occurs.
     */
    public static void setRankLevel(final int rank) throws IOException {
        OutputStream outputstream = null;
        try {
            outputstream = new FileOutputStream(CONFIG_HEAPSTATS);
            prop.setProperty("ranklevel", Integer.toString(rank));
            prop.store(outputstream, null);
            rankLevel = rank;
        } finally {
            if (outputstream != null) {
                outputstream.close();
            }
        }
    }

    /**
     * Returns the class path to rename.
     *
     * @return Returns the class path to rename.
     */
    public static boolean getClassRename() {
        return classRename;
    }

    /**
     * Return the List of log file to read.
     *
     * @return Return the List of log file to read
     */
    public static String getLogfiles() {
        return logfiles;
    }

    /**
     * Return the List of socket file.
     *
     * @return Return the List of socket file
     */
    public static String getSocketend() {
        return socketend;
    }

    /**
     * Return the Background color of the chart.
     *
     * @return Return the Background color of the chart
     */
    public static int getBgColor() {
        return bgColor;
    }

    /**
     * Return the resource.
     *
     * @return Return the resource
     */
    public static ResourceBundle getResources() {
        return resources;
    }

    /**
     * Return the current directory.
     *
     * @return Return the current directory
     */
    public static File getCurrentDir() {
        return currentDir;
    }

    /**
     * Sets the current directory.
     *
     * @param directory current directory
     * @throws IOException if writing this property list to the specified output
     *         stream throws an IOException.
     */
    public static void setCurrentDir(final File directory) throws IOException {
        OutputStream outputstream = null;
        try {
            currentDir = directory;
            outputstream = new FileOutputStream(CONFIG_HEAPSTATS);
            prop.setProperty("defaultdir", directory.getAbsolutePath());
            prop.store(outputstream, null);
        } finally {
            if (outputstream != null) {
                outputstream.close();
            }
        }
    }

    /**
     * Brings up a dialog that displays a message using a default icon
     * determined by the messageType parameter.
     *
     * @param component determines the Frame in which the dialog is displayed;
     *        if null, or if the parentComponent has no Frame, a default Frame
     *        is used
     * @param messageId the Object to display id
     * @param messageType the type of message to be displayed: ERROR_MESSAGE,
     *        INFORMATION_MESSAGE, WARNING_MESSAGE, QUESTION_MESSAGE, or
     *        PLAIN_MESSAGE
     */
    public static void showMessage(final Component component,
            final String messageId, final int messageType) {
        try {
            JOptionPane
                    .showMessageDialog(component,
                            resources.getString(messageId), ANALYZER_TITLE,
                            messageType);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Brings up a dialog that displays a message using a default icon
     * determined by the messageType parameter.
     *
     * @param component determines the Frame in which the dialog is displayed;
     *        if null, or if the parentComponent has no Frame, a default Frame
     *        is used
     * @param messageId the Object to display id
     * @param replacement The replacement sequence of char values
     * @param messageType the type of message to be displayed: ERROR_MESSAGE,
     *        INFORMATION_MESSAGE, WARNING_MESSAGE, QUESTION_MESSAGE, or
     *        PLAIN_MESSAGE
     */
    public static void showMessage(final Component component,
            final String messageId, final String replacement,
            final int messageType) {
        try {
            JOptionPane.showMessageDialog(
                    component,
                    resources.getString(messageId).replace(REPLACEMENT_CHAR,
                            replacement), ANALYZER_TITLE, messageType);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
