/*
 * SelectLogfiles.java
 * Created on 2012/01/24
 * RCSID: $Id: SelectLogfiles.java 385 2012-03-09 06:17:22Z alpha_takahiro.jyoukou $
 *
 * Copyright (C) 2011-2013 Nippon Telegraph and Telephone Corporation
 *
 */
package jp.co.ntt.oss.heapstats.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;

/**
 * extends {@link ExpansionContractionPanel}<br>
 * implements {@link ActionListener}<br>
 * <br>
 * The class reads the log file which the agent collected.
 */
public class SelectLogfiles extends ExpansionContractionPanel implements
                                                             ActionListener {
    /** serialVersionUID. */
    private static final long serialVersionUID = -2247677323518295406L;

    /** Resouce Data. */
    private ResourceDialog resourceDlg;

    /** Select Data Button. */
    private JButton logSelectBtn;

    /** Dump File Names. */
    private JTextField fileNames;

    /** Select Range StartDate. */
    private JComboBox startCombo;

    /** Select Range EndDate. */
    private JComboBox endCombo;

    /** Select Range OK Button. */
    private JButton okButton;

    /** The list of log files selected in the file chooser dialog. */
    private List<File> files;

    /** Key is Date. Data is the index of variable files & combo box. */
    private Map<String, Integer[]> logMap;

    /**
     * It is a screen part which reads the log file which the agent collected.
     *
     * @param title The title of an accordion panel
     * @param dialog Resource Data
     */
    public SelectLogfiles(final String title, final ResourceDialog dialog) {
        super(title);
        resourceDlg = dialog;

        addComponent();
    }

    @Override
    public final void actionPerformed(final ActionEvent actionevent) {
        Object eventSource = actionevent.getSource();
        if (!resourceDlg.isDone()) {
            // Being processed
            GuiCommon.showMessage(this, GuiCommon.BASEW02,
                    JOptionPane.WARNING_MESSAGE);
            return;
        }

        // SELECT DATA
        if (eventSource.equals(logSelectBtn)
                || actionevent.getActionCommand().equals(
                        GuiCommon.getResources().getString(
                                GuiCommon.MENU_FILE_LOGOPEN))) {
            loadLogfile();
        } else if (eventSource.equals(okButton)) {
            // Draws the Summary, Delta and SnapShot.
            int start = startCombo.getSelectedIndex();
            int end = endCombo.getSelectedIndex();

            if (selectRange(start, end)) {
                resourceDlg.draw(files, logMap, start, end);
            }
        }
    }

    /**
     * Create components to draw on the screen.
     */
    private void addComponent() {
        JPanel panelMain = new JPanel();
        panelMain.setLayout(new BorderLayout(GuiCommon.BORDER_SIZE,
                GuiCommon.BORDER_SIZE));
        panelMain.setBackground(Color.WHITE);

        logSelectBtn = new JButton(GuiCommon.getResources().getString(
                GuiCommon.BUTTON_LOG_SELECT));
        logSelectBtn.addActionListener(this);

        // SELECT FILE
        JPanel panelFile = new JPanel();
        panelFile.setLayout(new BorderLayout(GuiCommon.BORDER_SIZE,
                GuiCommon.BORDER_SIZE));
        panelFile.setBackground(Color.WHITE);

        fileNames = new JTextField();

        panelFile.add(logSelectBtn, BorderLayout.WEST);
        panelFile.add(fileNames, BorderLayout.CENTER);
        panelMain.add(panelFile, BorderLayout.NORTH);

        // Select Range
        JPanel panelRange = new JPanel();
        okButton = new JButton("OK");
        okButton.addActionListener(this);
        panelRange.setLayout(new BoxLayout(panelRange, BoxLayout.X_AXIS));
        panelRange.setBackground(Color.WHITE);

        startCombo = new JComboBox();
        startCombo.setEditable(true);

        JLabel llabel = new JLabel(" ~ ");
        endCombo = new JComboBox();
        endCombo.setEditable(true);

        panelRange.add(new JLabel(GuiCommon.getResources().getString(
                GuiCommon.SELECT_RANGE)
                + " "));
        panelRange.add(startCombo);
        panelRange.add(llabel);
        panelRange.add(endCombo);
        panelRange.add(Box.createVerticalStrut(GuiCommon.BORDER_SIZE));
        panelRange.add(okButton);

        panelMain.add(panelRange, BorderLayout.CENTER);
        panelRange.setLayout(new BoxLayout(panelRange, BoxLayout.X_AXIS));

        this.setComponent(panelMain, true);
    }

    /**
     * The log files which the agent collected is loaded.
     */
    private void loadLogfile() {
        JFileChooser fileDlg = getFileChooser();

        int result = fileDlg.showOpenDialog(this);

        if (result == JFileChooser.APPROVE_OPTION) {
            try {
                GuiCommon.setCurrentDir(fileDlg.getCurrentDirectory());
            } catch (IOException e) {
                GuiCommon.showMessage(this, GuiCommon.BASEE17,
                        JOptionPane.ERROR_MESSAGE);
            }

            files = new ArrayList<File>();
            logMap = new TreeMap<String, Integer[]>();

            resourceDlg.clear();

            startCombo.removeAllItems();
            endCombo.removeAllItems();
            fileNames.setText("");

            setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

            StringBuilder fileList = new StringBuilder();
            try {
                Vector<String> dateList = new Vector<String>();

                startCombo.removeAllItems();
                endCombo.removeAllItems();

                SimpleDateFormat sdf = new SimpleDateFormat(
                        GuiCommon.DATE_FORMAT);
                int fileIdx = 0;
                for (File file : fileDlg.getSelectedFiles()) {
                    File tempFile = File.createTempFile("heapstats", null);
                    tempFile.deleteOnExit();
                    FileOutputStream fstream = new FileOutputStream(
                            tempFile.getCanonicalPath(), false);
                    List<String> csvRows = new ArrayList<String>();

                    files.add(tempFile);

                    fileList.append(" ");
                    fileList.append(file.getName());
                    fileList.append(",");

                    BufferedReader read = new BufferedReader(new FileReader(
                            file));

                    String line = read.readLine();
                    while (line != null) {
                        csvRows.add(line);
                        // One line is read.
                        line = read.readLine();
                    }
                    read.close();

                    int rowIdx = 0;
                    Collections.sort(csvRows);
                    for (String row : csvRows) {
                        Integer[] data = new Integer[2];

                        data[0] = fileIdx;
                        data[1] = rowIdx++;

                        String[] rowSplit = row.split(",");

                        // key: collect date, data:file & rowIndex
                        logMap.put(rowSplit[0], data);

                        fstream.write(row.getBytes());
                        fstream.write(',');
                        fstream.write(file.getParent().getBytes());
                        fstream.write('\n');
                    }
                    fstream.close();

                    fileIdx++;
                }

                for (Map.Entry<String, Integer[]> entry : logMap.entrySet()) {
                    String date = sdf.format(Long.parseLong(entry.getKey()));
                    startCombo.addItem(date);
                    endCombo.addItem(date);
                    dateList.add(date);
                }

                startCombo.setSelectedIndex(0);
                endCombo.setSelectedIndex(endCombo.getItemCount() - 1);

                // KeyListener
                // Set SELECT RANGE combobox
                SnapshotDateListener start = new SnapshotDateListener(
                        startCombo);
                start.setSnapList(dateList);
                JTextField startTxt = (JTextField) startCombo.getEditor()
                        .getEditorComponent();
                startTxt.setText(startCombo.getItemAt(0).toString());
                startTxt.addKeyListener(start);

                SnapshotDateListener end = new SnapshotDateListener(endCombo);
                end.setSnapList(dateList);
                JTextField endTxt = (JTextField) endCombo.getEditor()
                        .getEditorComponent();
                endTxt.setText(endCombo.getItemAt(endCombo.getItemCount() - 1)
                        .toString());
                endTxt.addKeyListener(end);
                fileNames.setText(fileList.toString().substring(0,
                        fileList.length() - 1));
            } catch (Throwable e) {
                StringBuilder sb = new StringBuilder();
                for (File file : files) {
                    sb.append(file.getName());
                    sb.append(", ");
                }
                GuiCommon.showMessage(this, GuiCommon.BASEE08, sb.toString()
                        .substring(0, sb.length() - 2),
                        JOptionPane.ERROR_MESSAGE);
                fileList.setLength(0);
                files.clear();
                logMap.clear();
            } finally {
                setCursor(Cursor.getDefaultCursor());
            }
        }
    }

    /**
     * Return the FileChooser instance.
     *
     * @return Return the FileChooser instance
     */
    private JFileChooser getFileChooser() {
        JFileChooser fileDlg = new JFileChooser();
        fileDlg.setCurrentDirectory(GuiCommon.getCurrentDir());
        fileDlg.setMultiSelectionEnabled(true);
        fileDlg.setFileFilter(new FileFilter() {

            @Override
            public String getDescription() {
                return "CSV(*.csv, *.csv.*)";
            }

            @Override
            public boolean accept(final File f) {
                if (f.isDirectory()) {
                    return true;
                }

                String name = f.getName();
                int ext = name.lastIndexOf(".");

                while (ext > 0) {
                    if (name.substring(ext).toLowerCase().equals(".csv")) {
                        return true;
                    }
                    name = name.substring(0, ext);
                    ext = name.lastIndexOf(".");
                }
                return false;
            }
        });

        return fileDlg;
    }

    /**
     * Check the start and end snapshot of the selection period.
     *
     * @param start Start time of the snapshot for drawing
     * @param end End time of the snapshot for drawing
     * @return Return the check result.
     */
    private boolean selectRange(final int start, final int end) {
        if (start == -1 || end == -1) {
            GuiCommon.showMessage(this, GuiCommon.BASEE05,
                    JOptionPane.ERROR_MESSAGE);
            return false;
        } else if (start > end) {
            // Reversed start and end
            GuiCommon.showMessage(this, GuiCommon.BASEE06,
                    JOptionPane.ERROR_MESSAGE);
            return false;
        } else if (start == end) {
            // Match the start and end
            GuiCommon.showMessage(this, GuiCommon.BASEE07,
                    JOptionPane.ERROR_MESSAGE);
            return false;
        }
        return true;
    }
}
