/*
 * PackageFilterReader.java
 * Created on 2011/10/13
 * RCSID: $Id:$
 *
 * Copyright (C) 2011-2013 Nippon Telegraph and Telephone Corporation
 *
 */

package jp.co.ntt.oss.heapstats.gui.snapshot;

import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import jp.co.ntt.oss.heapstats.gui.GuiCommon;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;

/**
 * Read the file to parse filter definition.
 */
public class PackageFilterReader {
    /** filter file. */
    private File filterFile;

    /** Additional class information filtering. */
    private Map<String, List<String>> addFilter;

    /**
     * @param name a Filter definitions file
     */
    public PackageFilterReader(final File name) {
        filterFile = name;
    }

    /**
     * Load the filter definition file, and returns the results.
     *
     * @param owner the owner Dialog from which the dialog is displayed or null
     *        if this dialog has no owner
     * @return Return the Filter information
     */
    public final Map<String, List<String>> loadFilterFIle(final JDialog owner) {
        try {
            addFilter = new HashMap<String, List<String>>();
            SAXParserFactory saxFactory = SAXParserFactory.newInstance();
            saxFactory.setNamespaceAware(true);
            saxFactory.setValidating(true);
            SAXParser saxParser = saxFactory.newSAXParser();

            saxParser.setProperty(
                    "http://java.sun.com/xml/jaxp/properties/schemaLanguage",
                    "http://www.w3.org/2001/XMLSchema");
            saxParser.setProperty(
                    "http://java.sun.com/xml/jaxp/properties/schemaSource",
                    "file:filterDefine.xsd");

            ErrorHandler error = new ErrorHandler() {

                @Override
                public void warning(final SAXParseException exception)
                        throws SAXException {
                    addFilter = null;
                }

                @Override
                public void fatalError(final SAXParseException exception)
                        throws SAXException {
                    if (addFilter != null) {
                        JOptionPane.showMessageDialog(owner, GuiCommon
                                .getResources().getString(GuiCommon.BASEE11)
                                .replace("%s", exception.getMessage()),
                                GuiCommon.ANALYZER_TITLE,
                                JOptionPane.ERROR_MESSAGE);
                        addFilter = null;
                    }
                }

                @Override
                public void error(final SAXParseException exception)
                        throws SAXException {
                    if (addFilter != null) {
                        JOptionPane.showMessageDialog(owner, GuiCommon
                                .getResources().getString(GuiCommon.BASEE11)
                                .replace("%s", exception.getMessage()),
                                GuiCommon.ANALYZER_TITLE,
                                JOptionPane.ERROR_MESSAGE);
                        addFilter = null;
                    }
                }
            };

            ContentHandler handler = new ContentHandler() {

                /** Filter Class Path list. */
                private List<String> filterName;

                @Override
                public void startPrefixMapping(final String prefix,
                        final String uri) throws SAXException {
                }

                @Override
                public void startElement(final String uri,
                        final String localName, final String qName,
                        final Attributes atts) throws SAXException {

                    if (addFilter != null && qName.equals("filter")) {
                        filterName = new ArrayList<String>();
                        addFilter.put(atts.getValue(0), filterName);
                    }
                }

                @Override
                public void startDocument() throws SAXException {
                }

                @Override
                public void skippedEntity(final String name)
                        throws SAXException {
                }

                @Override
                public void setDocumentLocator(final Locator locator) {
                }

                @Override
                public void processingInstruction(final String target,
                        final String data) throws SAXException {
                }

                @Override
                public void ignorableWhitespace(final char[] ac, final int i,
                        final int j) throws SAXException {
                }

                @Override
                public void endPrefixMapping(final String prefix)
                        throws SAXException {
                }

                @Override
                public void endElement(final String uri,
                        final String localName, final String qName)
                        throws SAXException {
                }

                @Override
                public void endDocument() throws SAXException {
                }

                @Override
                public void characters(final char[] ac, final int start,
                        final int length) throws SAXException {
                    String name = new String(ac, start, length);

                    if (name.length() > 0 && filterName != null) {
                        if (name.substring(name.length() - 1).equals("*")) {
                            filterName.add(name.substring(0, name.length() - 1)
                                    + ".*");
                        } else {
                            filterName.add(name);
                        }
                    }
                }
            };

            XMLReader reader = saxParser.getXMLReader();

            reader.setContentHandler(handler);
            reader.setErrorHandler(error);

            reader.parse(new InputSource(new FileInputStream(filterFile
                    .getAbsolutePath())));
        } catch (Exception e) {
            if (addFilter != null) {
                JOptionPane.showMessageDialog(owner,
                        GuiCommon.getResources().getString(GuiCommon.BASEE11)
                                .replace("%s", e.getMessage()),
                        GuiCommon.ANALYZER_TITLE, JOptionPane.ERROR_MESSAGE);
                addFilter = null;
            }
        }

        return addFilter;
    }
}
